// C-Hack-Pannel CC-BY-SA-NC c-hack.de 2015
//
// Arduino Firmware for each Panel
// Last modification: 21-Jul-2015
// By: Ralf

// NOTE: Each panel has to have a different address between 1 and 6
#include "panel1.h"

// Define the 3 pins which drive the 74LS238  3:8 encoder
#define CA0  7     // ATMega168 Package Pin 13 
#define CA1  6     // ATMega168 Package Pin 12 
#define CA2  5     // ATMega168 Package Pin 11
                 
// This lookuptable maps the rows and bits to the respective IO pin of the ATMEGA168
// This is necessary because we optimized the layout for simple routing not for logical pin assignments :-)
//                Arduino IO     ATMega Package Pin #
int lookup[12] = {9,               //15
                  10,              //16
                  11,              //17
                  12,              //18
                  13,              //19
                  8,               //14
                  A0,              //23
                  A1,              //24
                  A2,              //25
                  A3,              //26
                  A4,              //27
                  A5};             //28
                  
int counter = 0;
int bitCounter = 1;
int loopCount = 0;

boolean recordIoData = false;
int ioCounter = 0;


// Runs only once at MCU power up
// Configures the IO Pins and the serial interface
void setup()
{
  Serial.begin(9600);
  for(int i = 0; i < 12; i++)
  {
     pinMode(lookup[i], OUTPUT); 
  }
  pinMode(CA0, OUTPUT); 
  pinMode(CA1, OUTPUT);  
  pinMode(CA2, OUTPUT);  
}

byte getCurrent(byte b, byte counter)
{
  return(b & counter);
}

void drawPanel()
{
 byte a0 = counter & 1;
 byte a1 = counter & 2;
 byte a2 = counter & 4;
 
 //  Set to 8th row in order to switch off the row drivers before changin the row content
 digitalWrite(CA0, 1);
 digitalWrite(CA1, 1);
 digitalWrite(CA2, 1);
 
 // Setting the row content in the order R1,G1,B1, R2,G2,B2, .... R5,G5,B5
 for(int i = 0; i <12; i++)
 {
   digitalWrite(lookup[11-i], getCurrent(data[i], bitCounter));
 }

 // Activating the current row again
 digitalWrite(CA0, a0);
 digitalWrite(CA1, a1);
 digitalWrite(CA2, a2);
 
 counter ++;
 bitCounter *= 2;
 
 if(counter >= 7) counter = 0;
 if(bitCounter >= 128) bitCounter = 1;
}

// Write a received byte into the data array
void processByte(byte b)
{
  if((b & 128) != 0) 
  {
    if((b & 127) == address) recordIoData = true;
  }
  else
  {
    if(recordIoData)
    {
      data[ioCounter] = b;
      ioCounter ++;
      if(ioCounter >= 13) 
      {
        recordIoData = false;
        ioCounter = 0;
      }
    }
  }
}

// Check if data is available on the serial interface and write into array
void checkIo()
{
  while (Serial.available() > 0) 
  {
   processByte(Serial.read()); 
  }
}

void loop()
{
 drawPanel();
 checkIo();
 delay(2);
 if (loopCount++ % 500 == 0) {
   Serial.print("C-Hack Panel #");
   Serial.println(address);
 }
}
