//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/
// PanelTest 1.0
// Ralf Stoffels - 2015
// 
// Hiermit können wir die Panel statisch ansprechen und so die 
// Kommunikation ausprobieren
//_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/_/


import processing.serial.*;
Serial myPort;  

int[][] panel = new int[][] {
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

int selectedColor = 0;

void setup() {
  size(1000, 400);
  background(0);
  println("Serial Interfaces found!");
  println(Serial.list());
  myPort = new Serial(this, "/dev/ttyUSB0", 115200);
  updatePanel();
  drawLED(0, 8, 1, false);
  drawLED(1, 8, 2, false);
  drawLED(2, 8, 4, false);
}

void draw() {
  // Da alles event driven ist, bleibt diese Funktion diesmal leer
  if (mousePressed) changeLED(); 
}

// Hier werden die LEDs gezeichnet und die Schaltflächen
void updatePanel() {
  drawBorders();
  for (int i=0; i<24; i++) 
     for (int j=0; j<7; j++) drawLED(i, j, panel[j][i], false);   
     
  // Hier noch die Schaltfläche für den Upload   
  fill(0,255,0);
  rect(440,350,120,20);
  textSize(15);
  fill(0);
  text("Upload",470,365);
}  

//Hier wird eine LED gezeichnet an der logischen Koorinate x,y
// Hierbei kann x zwichen 0 und 23 liegen und y zwischen 0 und 6 
void drawLED(int x, int y, int rgb, boolean hilite) {
  int r=0,b=0,g=0;
  if (hilite) {
     stroke(255);
     strokeWeight(3);
  } else {
     strokeWeight(4);
     stroke(0);
     ellipse(40+x*40,40+y*40,30,30);
     stroke(150);
     strokeWeight(1);
  }
  if ((rgb & 1) > 0) r=255;
  if ((rgb & 2) > 0) g=255;
  if ((rgb & 4) > 0) b=255;
  
  if (rgb>=0) fill(r,g,b); else noFill();
  ellipse(40+x*40,40+y*40,30,30);
}

// Diese Funktion zeichgnet die 6 Rechtecke der Panels
void drawBorders() {
  fill(0);
  stroke(255);
  strokeWeight(1);
  for (int i=0; i<6; i++) rect(20+160*i,20,160,300);
}

// Hier wird das komplette Panel hochgeladen zu den Ardunios
// Diese werden einzeln mit den Addressen 1..7 angesprochen
void uploadPanel() {
  int factor;
  int R, G, B;
  for (byte i=1; i<7; i++) {
    println("Upload to Panel ",i);
    myPort.write(i | 128); // Adresse schreiben höchstes Bit= 1
    
    for (int x=(i-1)*4+3; x>=(i-1)*4; x--) {
      R=0; G=0; B=0; factor=1;
      println(x);
      for (int y=6; y>=0; y--) {
        if ((panel[y][x] & 1) > 0) R = R + factor; 
        if ((panel[y][x] & 4) > 0) G = G + factor; 
        if ((panel[y][x] & 2) > 0) B = B + factor; 
        factor *= 2;
      }
      myPort.write(byte(R)); 
      myPort.write(byte(G));
      myPort.write(byte(B));
   
      println(R," ",G," ",B);
    }
  }
}

// Hier werden alle Maus-clicks verarbeitet (inklusive Upload Button) 
void changeLED() {
  int x,y;
  x = (mouseX-20)/40;
  y = (mouseY-20)/40;
  if ((x<24) && (y<7)) {
     panel[y][x] = selectedColor; 
     drawLED(x, y, panel[y][x], false);
  }
  if ((x==0) && (y==8)) {   // Rot geklickt
     selectedColor = selectedColor ^ 1;
     if ((selectedColor & 1) > 0) drawLED(x, y, 1, true); else drawLED(x, y, 1, false); 
  }
  if ((x==1) && (y==8)) {   // Grün geklickt
     selectedColor = selectedColor ^ 2;
     if ((selectedColor & 2) > 0) drawLED(x, y, 2, true); else drawLED(x, y, 2, false); 
  }
  if ((x==2) && (y==8)) {   // Blau geklickt
     selectedColor = selectedColor ^ 4;
     if ((selectedColor & 4) > 0) drawLED(x, y, 4, true); else drawLED(x, y, 4, false); 
  }
  if ((mouseX>440) && (mouseX<560) && (mouseY>350) && (mouseY<370)) uploadPanel();
}
